@@ -78,6 +78,12 @@ get_provider_name (GoaProvider *_provider,
   return g_strdup (_("Flickr"));
 }
 
+static GoaProviderGroup
+get_provider_group (GoaProvider *_provider)
+{
+  return GOA_PROVIDER_GROUP_BRANDED;
+}
+
 static const gchar *
 get_consumer_key (GoaOAuthProvider *provider)
 {
@@ -372,6 +378,7 @@ goa_flickr_provider_class_init (GoaFlickrProviderClass *klass)
   provider_class = GOA_PROVIDER_CLASS (klass);
   provider_class->get_provider_type     = get_provider_type;
   provider_class->get_provider_name     = get_provider_name;
+  provider_class->get_provider_group         = get_provider_group;
   provider_class->build_object          = build_object;
   provider_class->show_account          = show_account;
 
