@@ -86,6 +86,12 @@ get_provider_name (GoaProvider *provider, GoaObject *object)
   return g_strdup(_("Enterprise Login (Kerberos)"));
 }
 
+static GoaProviderGroup
+get_provider_group (GoaProvider *_provider)
+{
+  return GOA_PROVIDER_GROUP_TICKETING;
+}
+
 typedef struct
 {
   GtkDialog *dialog;
@@ -720,7 +726,7 @@ get_ticket_sync (GoaKerberosProvider *self,
     }
 
   memset (&request, 0, sizeof (SignInRequest));
-  request.loop = g_main_loop_new (NULL, FALSE);
+  request.loop = g_main_loop_new (g_main_context_get_thread_default (), FALSE);
   request.error = NULL;
 
   sign_in_identity (self,
@@ -754,12 +760,12 @@ notify_is_temporary_cb (GObject *object, GParamSpec *pspec, gpointer user_data)
   GoaAccount *account;
   gboolean is_temporary;
 
-  /* Toggle IsTemporary */
-  goa_util_account_notify_property_cb (object, pspec, "IsTemporary");
-
   account = GOA_ACCOUNT (object);
   g_object_get (account, "is-temporary", &is_temporary, NULL);
 
+  /* Toggle IsTemporary */
+  goa_utils_keyfile_set_boolean (account, "IsTemporary", is_temporary);
+
   /* Set/unset SessionId */
   if (is_temporary)
     {
@@ -1631,6 +1637,8 @@ start_over:
       goto start_over;
     }
 
+  goa_account_set_is_temporary (account, FALSE);
+
   gtk_widget_hide (GTK_WIDGET (dialog));
 
  out:
@@ -1779,9 +1787,6 @@ on_identity_looked_up_to_ensure_credentials (GoaKerberosProvider *self,
                     (GAsyncReadyCallback)
                     on_identity_looked_up,
                     operation_result);
-
-  g_simple_async_result_complete_in_idle (operation_result);
-  g_object_unref (operation_result);
 }
 
 static void
@@ -1884,6 +1889,7 @@ goa_kerberos_provider_class_init (GoaKerberosProviderClass *kerberos_class)
   provider_class = GOA_PROVIDER_CLASS (kerberos_class);
   provider_class->get_provider_type          = get_provider_type;
   provider_class->get_provider_name          = get_provider_name;
+  provider_class->get_provider_group         = get_provider_group;
   provider_class->build_object               = build_object;
   provider_class->add_account                = add_account;
   provider_class->refresh_account            = refresh_account;
