@@ -83,6 +83,12 @@ get_provider_name (GoaProvider *provider, GoaObject *object)
   return g_strdup(_("ownCloud"));
 }
 
+static GoaProviderGroup
+get_provider_group (GoaProvider *_provider)
+{
+  return GOA_PROVIDER_GROUP_BRANDED;
+}
+
 /* ---------------------------------------------------------------------------------------------------- */
 
 static gboolean on_handle_get_password (GoaPasswordBased      *interface,
@@ -340,9 +346,10 @@ ensure_credentials_sync (GoaProvider         *provider,
     {
       if (error != NULL)
         {
-          g_prefix_error (error, _("Did not find password with username `%s' in credentials"), username);
-          (*error)->domain = GOA_ERROR;
-          (*error)->code = GOA_ERROR_NOT_AUTHORIZED;
+          *error = g_error_new (GOA_ERROR,
+                                GOA_ERROR_NOT_AUTHORIZED,
+                                _("Did not find password with username `%s' in credentials"),
+                                username);
         }
       goto out;
     }
@@ -354,6 +361,25 @@ ensure_credentials_sync (GoaProvider         *provider,
                                     password,
                                     cancellable,
                                     error);
+  if (!ret)
+    {
+      if (error != NULL)
+        {
+          g_prefix_error (error,
+                          /* Translators: the first %s is the username
+                           * (eg., debarshi.ray@gmail.com or rishi), and the
+                           * (%s, %d) is the error domain and code.
+                           */
+                          _("Invalid password with username `%s' (%s, %d): "),
+                          username,
+                          g_quark_to_string ((*error)->domain),
+                          (*error)->code);
+          (*error)->domain = GOA_ERROR;
+          (*error)->code = GOA_ERROR_NOT_AUTHORIZED;
+        }
+      goto out;
+    }
+
   if (out_expires_in != NULL)
     *out_expires_in = 0;
 
@@ -563,7 +589,7 @@ create_account_details_ui (GoaProvider    *provider,
   gtk_container_add (GTK_CONTAINER (grid0), hbox);
 
   add_entry (grid1, grid2, _("_Server"), &data->uri);
-  add_entry (grid1, grid2, _("_User_name"), &data->username);
+  add_entry (grid1, grid2, _("User_name"), &data->username);
   add_entry (grid1, grid2, _("_Password"), &data->password);
   gtk_entry_set_visibility (GTK_ENTRY (data->password), FALSE);
 
@@ -945,6 +971,7 @@ goa_owncloud_provider_class_init (GoaOwncloudProviderClass *klass)
   provider_class = GOA_PROVIDER_CLASS (klass);
   provider_class->get_provider_type          = get_provider_type;
   provider_class->get_provider_name          = get_provider_name;
+  provider_class->get_provider_group         = get_provider_group;
   provider_class->add_account                = add_account;
   provider_class->refresh_account            = refresh_account;
   provider_class->build_object               = build_object;
