@@ -41,6 +41,7 @@
 #endif
 
 #include "goaeditablelabel.h"
+#include "goautils.h"
 
 /**
  * SECTION:goaprovider
@@ -170,6 +171,28 @@ goa_provider_get_provider_icon_default (GoaProvider *provider,
   return ret;
 }
 
+/**
+ * goa_provider_get_provider_group:
+ * @provider: A #GoaProvider.
+ *
+ * Gets the group to which @provider belongs that is suitable for
+ * organizing the providers while displaying them in an user
+ * interface.
+ *
+ * This is a pure virtual method - a subclass must provide an
+ * implementation.
+ *
+ * Returns: A #GoaProviderGroup.
+ *
+ * Since: 3.8
+ */
+GoaProviderGroup
+goa_provider_get_provider_group (GoaProvider *provider)
+{
+  g_return_val_if_fail (GOA_IS_PROVIDER (provider), GOA_PROVIDER_GROUP_INVALID);
+  return GOA_PROVIDER_GET_CLASS (provider)->get_provider_group (provider);
+}
+
 /* ---------------------------------------------------------------------------------------------------- */
 
 /**
@@ -881,56 +904,6 @@ goa_util_lookup_keyfile_boolean (GoaObject    *object,
 
 /* ---------------------------------------------------------------------------------------------------- */
 
-static void
-goa_util_set_keyfile_boolean (GoaAccount *account, const gchar *key, gboolean value)
-{
-  GError *error;
-  GKeyFile *key_file;
-  gchar *contents;
-  gchar *group;
-  gchar *path;
-  gsize length;
-
-  contents = NULL;
-
-  path = g_strdup_printf ("%s/goa-1.0/accounts.conf", g_get_user_config_dir ());
-  group = g_strdup_printf ("Account %s", goa_account_get_id (account));
-
-  key_file = g_key_file_new ();
-  error = NULL;
-  if (!g_key_file_load_from_file (key_file,
-                                  path,
-                                  G_KEY_FILE_KEEP_COMMENTS | G_KEY_FILE_KEEP_TRANSLATIONS,
-                                  &error))
-    {
-      goa_warning ("Error loading keyfile %s: %s (%s, %d)",
-                   path,
-                   error->message,
-                   g_quark_to_string (error->domain),
-                   error->code);
-      g_error_free (error);
-      goto out;
-    }
-
-  g_key_file_set_boolean (key_file, group, key, value);
-  contents = g_key_file_to_data (key_file, &length, NULL);
-
-  error = NULL;
-  if (!g_file_set_contents (path, contents, length, &error))
-    {
-      g_prefix_error (&error, "Error writing key-value-file %s: ", path);
-      goa_warning ("%s (%s, %d)", error->message, g_quark_to_string (error->domain), error->code);
-      g_error_free (error);
-      goto out;
-    }
-
- out:
-  g_free (contents);
-  g_key_file_free (key_file);
-  g_free (group);
-  g_free (path);
-}
-
 void
 goa_util_account_notify_property_cb (GObject *object, GParamSpec *pspec, gpointer user_data)
 {
@@ -946,7 +919,7 @@ goa_util_account_notify_property_cb (GObject *object, GParamSpec *pspec, gpointe
   name = g_param_spec_get_name (pspec);
 
   g_object_get (account, name, &value, NULL);
-  goa_util_set_keyfile_boolean (account, key, !value);
+  goa_utils_keyfile_set_boolean (account, key, !value);
 }
 
 /* ---------------------------------------------------------------------------------------------------- */
